package net.indiespot.sql.data;

import log.Log;
import java.util.List;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.CopyOnWriteArraySet;
import java.util.concurrent.atomic.AtomicBoolean;


public interface RecordManagerDomain
{
  public <T extends Record<T>> Class<T> lookup(String simpleName);

  public <T extends Record<T>> T newRecord(Class<T> type);

  public void resolveInboundForeignKeys(Fetcher f);

  public static final class Access
  {
    private static final Object MUTEX = new Object();
    private static final Set<RecordManagerDomain> DOMAIN_SET;
    private static final List<RecordManagerDomain> DOMAIN_LIST;
    private static boolean ACCESSED;

    static {
      DOMAIN_SET = new CopyOnWriteArraySet<>();
      DOMAIN_LIST = new CopyOnWriteArrayList<>();
      ACCESSED = false;
    }

    public static void register(RecordManagerDomain domain)
    {
      if (ACCESSED)
        throw new IllegalStateException(
            "cannot register app-contexts when already accessed");

      synchronized (MUTEX) {
        if (!DOMAIN_SET.add(domain))
          throw new IllegalStateException("duplicate project domain");
        DOMAIN_LIST.add(domain);
      }
    }

    private static AtomicBoolean HAS_WARNED_ABOUT_NO_DOMAINS =
      new AtomicBoolean();

    public static List<RecordManagerDomain> get()
    {
      ACCESSED = true;
      if (DOMAIN_LIST.isEmpty()
          && HAS_WARNED_ABOUT_NO_DOMAINS.compareAndSet(false, true)) {
        Log.warn("no domains defined");
      }
      return DOMAIN_LIST;
    }
  }
}
