package net.indiespot.sql.data;

import net.indiespot.sql.ResultSetRowVisitor;
import nl.west.rme.common.util.Text;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;


public class RecordQuery
{
  private final StringBuilder query;
  private final Map<String, Object> params;

  public RecordQuery()
  {
    query = new StringBuilder();
    params = new HashMap<>();
  }

  public RecordQuery withSQL(String q)
  {
    query.append(q).append("\r\n");
    return this;
  }

  public RecordQuery clearParam(String prop)
  {
    prop = Text.trimToNull(prop);
    if (prop == null)
      throw new IllegalArgumentException();
    if (!params.containsKey(prop))
      throw new IllegalStateException("missing parameter: '" + prop + "'");
    params.remove(prop);
    return this;
  }

  public RecordQuery withParam(String prop, Object val)
  {
    prop = Text.trimToNull(prop);
    if (prop == null)
      throw new IllegalArgumentException();
    if (params.containsKey(prop))
      throw new IllegalStateException("duplicate parameter: '" + prop + "'");
    params.put(prop, val);
    return this;
  }

  public int query(ResultSetRowVisitor rowVisitor)
  {
    return JdbcRecordManager.query(query.toString(), params, rowVisitor);
  }

  public <T extends Record<T>> Set<T> query(Class<T> type)
  {
    return JdbcRecordManager.query(type, query.toString(), params);
  }

  public <T extends Record<T>> List<T> queryOrdered(Class<T> type)
  {
    return JdbcRecordManager.queryOrdered(type, query.toString(), params);
  }

  public <T extends Record<T>> Set<Key<T>> queryKeys(Class<T> keyType)
  {
    return JdbcRecordManager.queryKeys(query.toString(), params, keyType);
  }

  public Fetcher query(Class<?>... types)
  {
    return JdbcRecordManager.query(types, query.toString(), params);
  }

  public int queryInt()
  {
    return JdbcRecordManager.queryInt(query.toString(), params);
  }

  public <T extends Record<T>> T queryFirstIfAny(Class<T> type)
  {
    return JdbcRecordManager.queryFirstIfAny(type, query.toString(), params);
  }

  public String debug()
  {
    QueryRewriter qr;
    qr = new QueryRewriter(query.toString());
    qr.rewrite(params);
    return qr.debug();

  }
}
