package net.indiespot.sql.data;

import nl.west.rme.common.util.Pair;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


public class RecordTypeMetadata
{
  public static <T extends Record<T>> int columnCountForSelect(Class<T> type)
  {
    return getDataFields(type).size();
  }

  public static <T extends Record<T>> List<String> getColumnNames(
      Class<T> type, boolean withPK)
  {
    final List<String> columnNames = new ArrayList<>();
    for (final Field field : getDataFields(type)) {
      if (!withPK)
        if (field.getName().equals("id"))
          continue;
      columnNames.add(field.getName());
    }
    return columnNames;
  }

  private static final Map<Pair<Class<?>, String>, String> TYPE_ALIAS_2_STMT =
    new HashMap<>();

  public static <T extends Record<T>> String columnNamesForSelect(
      Class<T> type, String alias)
  {
    synchronized (TYPE_ALIAS_2_STMT) {
      final Pair<Class<?>, String> key =
        new Pair<Class<?>, String>(type, alias);
      String stmt = TYPE_ALIAS_2_STMT.get(key);
      if (stmt == null)
        TYPE_ALIAS_2_STMT.put(key, stmt = columnNamesForSelect0(type, alias));

      return stmt;
    }
  }

  private static <T extends Record<T>> String columnNamesForSelect0(
      Class<T> type, String alias)
  {
    final StringBuilder sb = new StringBuilder();
    for (final String columnName : RecordTypeMetadata
        .getColumnNames(type, true)) {
      if (sb.length() > 0)
        sb.append(", ");
      if (alias != null)
        sb.append(alias).append(".");
      sb.append(columnName);
    }
    return sb.toString();
  }

  // --------------

  private static final Map<Class<?>, Field> TYPE_2_PK_FIELD = new HashMap<>();
  private static final Map<Class<?>, List<Field>> TYPE_2_KEY_FIELDS =
    new HashMap<>();
  private static final Map<Class<?>, List<Field>> TYPE_2_DATA_FIELDS =
    new HashMap<>();

  public static <T extends Record<T>> Field getPrimaryKeyField(Class<T> type)
  {
    synchronized (TYPE_2_PK_FIELD) {
      Field pkField = TYPE_2_PK_FIELD.get(type);
      if (pkField == null) {
        for (final Field dataField : RecordTypeMetadata.getDataFields(type))
          if (dataField.getName().equals("id"))
            pkField = dataField;
        if (pkField == null)
          throw new IllegalStateException("primary key field not found");
        TYPE_2_PK_FIELD.put(type, pkField);
      }
      return pkField;
    }
  }

  // --------------

  public static <T extends Record<T>> List<Field> getKeyFields(Class<T> type)
  {
    synchronized (TYPE_2_KEY_FIELDS) {
      List<Field> fields = TYPE_2_KEY_FIELDS.get(type);
      if (fields == null) {
        fields = new ArrayList<>();
        for (final Field field : getDataFields(type))
          if (field.getType() == Key.class)
            fields.add(field);
        TYPE_2_KEY_FIELDS.put(type, Collections.unmodifiableList(fields));
      }
      return fields;
    }
  }

  public static <T extends Record<T>> List<Field> getDataFields(Class<T> type)
  {
    synchronized (TYPE_2_DATA_FIELDS) {
      List<Field> fields = TYPE_2_DATA_FIELDS.get(type);
      if (fields == null) {
        Field pkField = null;

        fields = new ArrayList<>();
        for (final Field field : type.getFields())
          if (!Modifier.isStatic(field.getModifiers()))
            if (!Modifier.isTransient(field.getModifiers()))
              if (field.getName().equals("id"))
                pkField = field;
              else
                fields.add(field);

        fields.add(0, pkField);

        TYPE_2_DATA_FIELDS.put(
            type, fields = Collections.unmodifiableList(fields));
      }
      return fields;
    }
  }
}
