package net.indiespot.sql.dialects;

import log.Log;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

public class MySqlConnectionConfiguration {
	private static final String charset = "utf8mb4";
	private static final String collation = "utf8mb4_unicode_ci";

	public static void useDatabase(Connection conn, String dbName) throws SQLException {
		try (Statement stmt = conn.createStatement()) {
			final String query = "USE `" + dbName + "`";
			stmt.executeUpdate(query);
		} catch (final SQLException e) {
			try (Statement stmt = conn.createStatement()) {
				String query = "CREATE DATABASE `" + dbName + "`";
				query += " DEFAULT CHARACTER SET '" + charset + "'";
				query += " DEFAULT COLLATE '" + collation + "'";
				stmt.execute(query);
			} catch (final SQLException e2) {
				throw new IllegalStateException(e2);
			}

			try (Statement stmt = conn.createStatement()) {
				final String query = "USE `" + dbName + "`";
				stmt.executeUpdate(query);
			}
		}
	}

	public static void configure(Connection conn, boolean verbose) throws SQLException {
		if (verbose) {
			try (Statement stmt = conn.createStatement()) {
				try (ResultSet result = stmt.executeQuery("SELECT @@version")) {
					while (result.next()) {
						Log.debug("MySQL server version: " + result.getString(1));
					}
				}
			}

			Log.debug("MySQL client version: " + conn.getMetaData().getDriverVersion());
		}

		try (Statement stmt = conn.createStatement()) {
			stmt.execute("SET character_set_server = '" + charset + "'");
			stmt.execute("SET collation_server     = '" + collation + "'");
			stmt.execute("SET collation_connection = '" + collation + "'");
			stmt.execute("SET NAMES " + charset);

			try (ResultSet result = stmt.executeQuery("SHOW VARIABLES LIKE 'character_set_server'")) {
				if (!result.next())
					throw new IllegalStateException();
				if (!charset.equalsIgnoreCase(result.getString(2)))
					throw new IllegalStateException("server/client charset mismatch: got=[" + result.getString(2)
							+ "], required=[" + charset + "]");
				if (result.next())
					throw new IllegalStateException();
			}

			try (ResultSet result = stmt.executeQuery("SHOW VARIABLES LIKE 'collation_server'")) {
				if (!result.next())
					throw new IllegalStateException();
				if (!collation.equalsIgnoreCase(result.getString(2)))
					throw new IllegalStateException("server/client charset mismatch: got=[" + result.getString(2)
							+ "], required=[" + collation + "]");
				if (result.next())
					throw new IllegalStateException();
			}
		}
	}
}
