package nl.billinghouse.rme;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.io.FileUtils;

import log.Log;
import nl.billinghouse.rme.dao.Build;
import nl.billinghouse.rme.dao.Portal;
import nl.billinghouse.rme.utility.Streams;
import nl.billinghouse.rme.utility.Text;

public class BuildService {
	static String sshPath = "C:/Users/Skip/work/west/plink.exe";
	static String scpPath = "C:/Users/Skip/work/west/pscp.exe";
	static String remoteHost = "indiespot.net";
	static int remotePort = 8192;
	static String remoteUser = "root";
	static String remoteBasePath = "/home/websites/domains/west.indiespot.net";
	static String authKeyPath = "C:/Users/Skip/work/west/id_rsa.ppk";

	public static void build(Build build) {
		Log.info("Building: " + build);

		String relPath = "manui-cdn/build-" + build.id;
		File buildDir = new File("C:/Users/Skip/work/west/apache-www", relPath);

		String srcPath = buildDir.getAbsolutePath() + "/";
		String dstPath = remoteBasePath + "/" + relPath + "/";

		build.file_root = srcPath;
		build.http_root = "http://west.indiespot.net/" + relPath;
		build.update();

		copyResourceDir("assets/manui-html5", buildDir);

		replaceFileContent(//
				new File(buildDir, "app/manui-context-build.js"), //
				"'[[suitecrm_model]]'", //
				build.suitecrm_layout_json);

		// replaceFileContent(//
		// new File(targetDir, "app/manui-context.js"), //
		// "'[[portal_id]]'", //
		// build.suitecrm_layout_json);

		replaceFileContent(//
				new File(buildDir, "app/form.html"), //
				"[[build_http_root]]", //
				build.http_root);
		replaceLineContaining(//
				new File(buildDir, "app/form.html"), //
				"[[portal_style_url]]", //
				"");

		replaceFileContent(//
				new File(buildDir, "app/import.html"), //
				"[[build_http_root]]", //
				build.http_root);

		replaceFileContent(//
				new File(buildDir, "app/import.html"), //
				"[[portal_http_root]]", //
				build.http_root);

		{
			ssh(sshPath, remoteHost, remotePort, remoteUser, authKeyPath, //
					"mkdir -p " + dstPath);

			scp(scpPath, remoteHost, remotePort, remoteUser, authKeyPath, srcPath, dstPath);
		}
	}

	public static void publish(Portal portal, Build build) {
		Log.info("Publishing: " + portal);

		String relPath = "manui-cdn/portal-" + portal.id;
		File resourceDir = getResourceAsFile("assets/manui-html5");
		File portalDir = new File("C:/Users/Skip/work/west/apache-www", relPath);
		File buildDir = new File(build.file_root);
		
		String srcPath = portalDir.getAbsolutePath() + "/";
		String dstPath = remoteBasePath + "/" + relPath + "/";

		portal.build = build.getKey();
		portal.http_root = "http://west.indiespot.net/" + relPath;
		portal.update();

		{
			try {
				FileUtils.copyDirectory(buildDir, portalDir);
			} catch (IOException e) {
				throw new IllegalStateException(e);
			}
		}

		{
			if (portal.style_url == null) {
				replaceLineContaining(//
						new File(portalDir, "app/form.html"), //
						"[[portal_style_url]]", //
						"");
			} else {
				replaceFileContent(//
						new File(portalDir, "app/form.html"), //
						"[[portal_style_url]]", //
						portal.style_url);
			}

			File resourceImport = new File(resourceDir, "app/import.html");
			File portalImport = new File(portalDir, "app/import.html");
			try {
				FileUtils.copyFile(resourceImport, portalImport);
			} catch (IOException e) {
				throw new IllegalStateException(e);
			}

			replaceFileContent(//
					portalImport, //
					"[[build_http_root]]", //
					build.http_root);

			replaceFileContent(//
					portalImport, //
					"[[portal_http_root]]", //
					portal.http_root);
		}

		{
			File resourceContext = new File(resourceDir, "app/manui-context-portal.js");
			File portalContext = new File(portalDir, "app/manui-context-portal.js");
			try {
				FileUtils.copyFile(resourceContext, portalContext);
			} catch (IOException e) {
				throw new IllegalStateException(e);
			}

			replaceFileContent(//
					portalContext, //
					"[[portal_id]]", //
					String.valueOf(portal.name)); // TODO: escape
		}
		
		{
			ssh(sshPath, remoteHost, remotePort, remoteUser, authKeyPath, //
					"mkdir -p " + dstPath);

			scp(scpPath, remoteHost, remotePort, remoteUser, authKeyPath, srcPath, dstPath);
		}
	}

	private static File getResourceAsFile(String resourcePath) {
		return new File(BuildService.class.getClassLoader().getResource(resourcePath).getPath());
	}

	private static void copyResourceFile(String resourcePath, File dst) {
		final File src = getResourceAsFile(resourcePath);

		try {
			FileUtils.copyFile(src, dst);
		} catch (final IOException e) {
			throw new IllegalStateException("Error copying files", e);
		}
	}

	private static void copyResourceDir(String resourcePath, File dst) {
		final File src = getResourceAsFile(resourcePath);

		try {
			FileUtils.copyDirectory(src, dst);
		} catch (final IOException e) {
			throw new IllegalStateException("Error copying files", e);
		}
	}

	private static void replaceFileContent(File file, String find, String replaceWith) {
		try {
			final Charset charset = Charset.forName("UTF-8");
			final Path path = Paths.get(file.getAbsolutePath());

			byte[] raw = Files.readAllBytes(path);
			String txt = new String(raw, charset);
			{
				txt = txt.replace(find, replaceWith);
			}
			raw = txt.getBytes(charset);

			Files.write(path, raw);
		} catch (final IOException e) {
			throw new IllegalStateException(e);
		}
	}

	private static void replaceLineContaining(File file, String find, String replaceWith) {
		try {
			final Charset charset = Charset.forName("UTF-8");
			final Path path = Paths.get(file.getAbsolutePath());

			byte[] raw = Files.readAllBytes(path);
			String txt = new String(raw, charset);
			{
				StringBuilder out = new StringBuilder();
				for (String line : txt.split("(\\r\\n)|(\\r)|(\\n)")) {
					if (line.contains(find))
						line = replaceWith;
					out.append(line).append("\n");
				}
				txt = out.toString();
			}
			raw = txt.getBytes(charset);

			Files.write(path, raw);
		} catch (final IOException e) {
			throw new IllegalStateException(e);
		}
	}

	private static void ssh(String sshPath, //
			String sshHost, int sshPort, String sshUser, String sshKeyPath, //
			String command) {

		byte[] stdout = null;
		String keyChecking = null;

		if (System.getProperty("os.name").contains("Windows")) {
			stdout = Text.utf8("y\n"); // make sure pscp accepts key
		} else {
			keyChecking = "-oStrictHostKeyChecking=no";
		}

		exec(stdout, //
				sshPath, //
				keyChecking, //
				"-P", String.valueOf(sshPort), //
				"-i", sshKeyPath, //
				"-l", sshUser, //
				sshHost, command);
	}

	private static void scp(String scpPath, //
			String scpHost, int scpPort, String scpUser, String scpKeyPath, //
			String srcPath, String dstPath) {

		byte[] stdout = null;
		String keyChecking = null;

		if (System.getProperty("os.name").contains("Windows")) {
			stdout = Text.utf8("y\n"); // make sure pscp accepts key
		} else {
			keyChecking = "-oStrictHostKeyChecking=no";
		}

		exec(stdout, //
				scpPath, //
				keyChecking, //
				"-p", //
				"-P", String.valueOf(scpPort), //
				"-i", scpKeyPath, //
				"-r", srcPath, //
				scpUser + "@" + scpHost + ":" + dstPath);
	}

	private static void exec(byte[] stdout, String... args) {
		System.out.println(Arrays.toString(args));
		try {
			List<String> list = new ArrayList<>();
			for (String arg : args)
				if (arg != null)
					list.add(arg);

			final Process p = Runtime.getRuntime().exec(list.toArray(new String[list.size()]));

			for (final InputStream std : new InputStream[] { p.getInputStream(), p.getErrorStream() }) {
				new Thread(new Runnable() {
					@Override
					public void run() {
						try {
							Streams.copy(std, System.out);
						} catch (final IOException e) {
							e.printStackTrace();
						}
					}
				}).start();
			}

			if (stdout != null) {
				OutputStream out = p.getOutputStream();
				out.write(stdout);
				out.flush();
				out.close();
			}

			final int exitCode = p.waitFor();
			System.out.println("exit-code:" + exitCode);
		} catch (final Exception exc) {
			exc.printStackTrace();
		}
	}
}
