package nl.billinghouse.rme;

import java.io.IOException;
import java.util.Date;
import java.util.List;
import java.util.Set;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;

import log.Log;
import net.indiespot.sql.data.Fetcher;
import net.indiespot.sql.data.JdbcRecordManager;
import net.indiespot.sql.data.JsonRecordManager;
import net.indiespot.sql.data.Key;
import net.indiespot.sql.data.RecordQuery;
import nl.billinghouse.rme.dao.Build;
import nl.billinghouse.rme.dao.Portal;
import nl.billinghouse.rme.utility.JsonIO;
import nl.billinghouse.rme.utility.JsonServlet;
import nl.west.rme.common.Hash;

public class BuildServlet extends JsonServlet {
	@Override
	protected void onBeforeService(HttpServletRequest req, HttpServletResponse resp) {
		super.onBeforeService(req, resp);

		try {
			Thread.sleep(500 + (long) (Math.random() * 500));
		} catch (InterruptedException exc) {

		}
	}

	public JsonElement doGetAll(JsonElement elem, HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {

		List<Build> builds;
		{
			RecordQuery rq = new RecordQuery();
			rq.withSQL("SELECT Build.*        ");
			rq.withSQL("  FROM Build          ");
			rq.withSQL(" ORDER BY id DESC     ");
			builds = rq.queryOrdered(Build.class);
		}

		Fetcher f = new Fetcher();
		f.addAll(builds);
		f.addAll(JdbcRecordManager.queryAll(Portal.class));
		f.resolve();

		return JsonRecordManager.toJsonArray(builds, new JsonRecordManager.JsonTransformer<Build>() {
			public void transform(Build record, JsonObject obj) {
				obj.add("suitecrm_layout_json", null);
				obj.addProperty("portal_usage_count", record.$meta.portals.size());
			}
		});
	}

	public JsonElement doGetByCategory(JsonElement elem, HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {

		String category = getPath(req)[1];

		List<Build> builds;
		{
			RecordQuery rq = new RecordQuery();
			rq.withSQL("SELECT Build.*        ");
			rq.withSQL("  FROM Build          ");
			rq.withSQL(" WHERE category = :category ");
			rq.withSQL(" ORDER BY id DESC     ");
			rq.withParam("category", category);
			builds = rq.queryOrdered(Build.class);
		}

		return JsonRecordManager.toJsonArray(builds, new JsonRecordManager.JsonTransformer<Build>() {
			public void transform(Build record, JsonObject obj) {
				obj.add("suitecrm_layout_json", null);
			}
		});
	}

	public JsonElement doGetFetch(JsonElement elem, HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {

		String categoryName = "Deployable";
		String categoryUUID = "70662b81-ea25-82c5-b889-56543b0a5987";

		JsonObject teamModel = SuiteCRM.getTeamModel(categoryUUID);
		String hash = Hash.utf8sha1hex(JsonIO.toJson(teamModel));

		RecordQuery rq = new RecordQuery();
		rq.withSQL("SELECT Build.*");
		rq.withSQL("  FROM Build");
		rq.withSQL(" WHERE category = :category");
		rq.withSQL("   AND suitecrm_layout_sha1 = :hash");
		rq.withParam("category", categoryName);
		rq.withParam("hash", hash);
		final Build found = rq.queryFirstIfAny(Build.class);

		boolean isNewBuild = (found == null);
		Log.info("isNewBuild: " + isNewBuild);

		if (isNewBuild) {
			Build build = new Build();
			build.name = "Build -1";
			build.category = categoryName;
			build.moment = System.currentTimeMillis();
			build.suitecrm_layout_sha1 = hash;
			build.suitecrm_layout_json = JsonIO.toJson(teamModel);
			build.insert();

			build.name = "Build " + build.id;
			build.update();

			build = build.getKey().select();
			BuildService.build(build);

			// BuildService.publish(Key.create(Portal.class, 1).select(),
			// build);
			// BuildService.publish(Key.create(Portal.class, 2).select(),
			// build);
			// BuildService.publish(Key.create(Portal.class, 3).select(),
			// build);
			// BuildService.publish(Key.create(Portal.class, 4).select(),
			// build);
		}

		return new JsonPrimitive(isNewBuild);
	}
}
