package nl.billinghouse.rme;

import java.io.IOException;
import java.util.NoSuchElementException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.google.gson.JsonElement;
import com.google.gson.JsonNull;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;

import net.indiespot.sql.data.JsonRecordManager;
import net.indiespot.sql.data.Key;
import net.indiespot.sql.data.Record;
import net.indiespot.sql.data.RecordManager;
import nl.billinghouse.rme.utility.JsonServlet;
import nl.billinghouse.rme.utility.Text;

public class CrudServlet extends JsonServlet {

	public <T extends Record<T>> JsonElement doGet(JsonElement elem, HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		String[] path = getPath(req);
		String simpleName = Text.capitalize(path[0]);
		int id = Integer.parseInt(path[1]);

		Class<T> type = RecordManager.lookupTypeByName(simpleName);
		Key<T> key = Key.create(type, id);
		T record = key.selectOrNull();
		return JsonRecordManager.toJson(record);
	}

	public <T extends Record<T>> JsonElement doPut(JsonElement elem, HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		String[] path = getPath(req);
		String simpleName = Text.capitalize(path[0]);

		Class<T> type = RecordManager.lookupTypeByName(simpleName);

		T toInsert = JsonRecordManager.fromJson(type, (JsonObject) elem);
		toInsert.insert();

		T normalized = toInsert.getKey().select();
		return JsonRecordManager.toJson(normalized);
	}

	public <T extends Record<T>> JsonElement doPost(JsonElement elem, HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		String[] path = getPath(req);
		String simpleName = Text.capitalize(path[0]);
		int id = Integer.parseInt(path[1]);

		Class<T> type = RecordManager.lookupTypeByName(simpleName);
		Key<T> key = Key.create(type, id);
		T current = key.selectOrNull();
		if (current == null)
			throw new NoSuchElementException();

		T toUpdate = JsonRecordManager.fromJson(type, (JsonObject) elem);
		if (toUpdate.id != current.id)
			throw new IllegalStateException();
		toUpdate.update();

		T normalized = toUpdate.getKey().select();
		return JsonRecordManager.toJson(normalized);
	}

	public <T extends Record<T>> JsonElement doDelete(JsonElement elem, HttpServletRequest req,
			HttpServletResponse resp) throws ServletException, IOException {
		String[] path = getPath(req);
		String simpleName = Text.capitalize(path[0]);
		int id = Integer.parseInt(path[1]);

		Class<T> type = RecordManager.lookupTypeByName(simpleName);
		Key<T> key = Key.create(type, id);
		T current = key.selectOrNull();
		if (current == null)
			return new JsonPrimitive(false);

		current.delete();

		return new JsonPrimitive(true);
	}
}
