package nl.billinghouse.rme.utility;

import com.google.gson.JsonElement;
import com.google.gson.JsonPrimitive;
import java.io.IOException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


@SuppressWarnings("serial")
public class JsonServlet extends BaseServlet
{
  /**
   * POST /hello-world/bye-sky <br/>
   * JsonElement result = doPostHelloWorld(JsonElement, req, resp)
   */

  protected static final JsonElement SIGNAL_EMPTY_RESPONSE_BODY =
    new JsonPrimitive(Boolean.FALSE);

  @Override
  protected Class<?>[] getMethodParameters()
  {
    return new Class[] {
        JsonElement.class, HttpServletRequest.class, HttpServletResponse.class };
  }

  @Override
  protected Object[] getMethodArguments(
      HttpServletRequest req, HttpServletResponse resp)
  {
    String json;
    try {
      json =
        Text.trimToNull(Text.utf8(Streams.readFully(req.getInputStream())));
    }
    catch (final IOException e) {
      json = null;
    }

    if (json == null)
      json = req.getParameter("json");

    final JsonElement elem = (json == null) ? null : JsonIO.fromJson(json);

    return new Object[] { elem, req, resp };
  }

  @Override
  protected void onSuccessHandler(
      Object result, HttpServletRequest req, HttpServletResponse resp)
      throws IOException
  {
    final String json =
      (result == SIGNAL_EMPTY_RESPONSE_BODY) ? "" : JsonIO
          .toJson((JsonElement) result, true);

    resp.setStatus(HttpServletResponse.SC_OK);
    resp.setContentType("application/json");
    resp.setCharacterEncoding("UTF-8");

    resp.getWriter().println(json);
  }
}
