package nl.billinghouse.rme.utility;

import org.apache.commons.io.IOUtils;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;


public class Streams
{
  public static void writeFile(File file, byte[] data)
      throws IOException
  {
    try (FileOutputStream fos = new FileOutputStream(file)) {
      fos.write(data);
    }
  }

  public static byte[] readFile(File file)
      throws IOException
  {
    return readFully(new FileInputStream(file));
  }

  public static byte[] readFully(InputStream in)
      throws IOException
  {
    final ByteArrayOutputStream baos = new ByteArrayOutputStream();
    copy(in, baos);
    return baos.toByteArray();
  }

  public static void copy(InputStream in, OutputStream out)
      throws IOException
  {
    try {
      final byte[] tmp = new byte[4 * 1024];
      while (true) {
        final int got = in.read(tmp);
        if (got == -1)
          break;
        out.write(tmp, 0, got);
      }
    }
    finally {
      in.close();
    }
  }

  public static byte[] compressToZip(String folder)
      throws IOException
  {
    final File directory = new File(folder);
    if (!directory.isDirectory() || directory.list().length == 0)
      throw new IllegalStateException("must compress zip non-empty directory");

    final ByteArrayOutputStream baos = new ByteArrayOutputStream();

    final List<File> remaining = new ArrayList<>();
    remaining.add(directory);

    final String basePath = directory.getAbsolutePath();

    try (final ZipOutputStream zos = new ZipOutputStream(baos)) {
      while (!remaining.isEmpty()) {
        final File file = remaining.remove(0);
        if (file.isDirectory()) {
          for (final File child : file.listFiles())
            remaining.add(child);
          continue;
        }

        final String filePath = file.getAbsolutePath();
        if (!filePath.startsWith(basePath))
          throw new IllegalStateException("left base directory");
        String relPath = filePath.substring(basePath.length());
        while (relPath.startsWith("/") || relPath.startsWith("\\"))
          relPath = relPath.substring(1);

        final ZipEntry entry = new ZipEntry(relPath);
        zos.putNextEntry(entry);

        try (final FileInputStream fis = new FileInputStream(file)) {
          IOUtils.copy(fis, zos);
        }
        zos.closeEntry();
      }
      zos.close();
      return baos.toByteArray();
    }
  }

  public static void extractZip(byte[] zip, String folder)
      throws IOException
  {
    final File directory = new File(folder);
    if (directory.isDirectory() && directory.list().length > 0)
      throw new IllegalStateException("must extract zip to empty directory");

    try (final ZipInputStream zis =
      new ZipInputStream(new ByteArrayInputStream(zip))) {
      while (true) {
        final ZipEntry entry = zis.getNextEntry();
        if (entry == null)
          break;

        final String relpath = entry.getName();
        final File out = new File(folder + "/" + relpath);
        out.getParentFile().mkdirs();

        try (final FileOutputStream fos = new FileOutputStream(out)) {
          IOUtils.copy(zis, fos);
        }
      }
    }
  }
}
