package nl.west.rme.common;

import nl.west.rme.common.util.Text;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.DigestOutputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

public class Hash {
	public static byte[] sha1(byte[] data) {
		return sha1(data, 0, data.length);
	}

	public static byte[] sha1(byte[] data, int off, int len) {
		return hash(data, off, len, "SHA-1");
	}

	public static String utf8sha1hex(String txt) {
		return Text.toHex(sha1(Text.utf8(txt)));
	}

	// ----

	public static byte[] sha256(byte[] data) {
		return sha256(data, 0, data.length);
	}

	public static byte[] sha256(byte[] data, int off, int len) {
		return hash(data, off, len, "SHA-256");
	}

	public static String utf8sha256hex(String txt) {
		return Text.toHex(sha256(Text.utf8(txt)));
	}

	public static String utf8sha256hex(String a, String b) {
		return utf8sha256hex(utf8sha256hex(a) + ":" + utf8sha256hex(b));
	}

	// ----

	public static byte[] hash(byte[] data, int off, int len, String algorithm) {
		try {
			final MessageDigest digest = MessageDigest.getInstance(algorithm);
			final ByteArrayOutputStream baos = new ByteArrayOutputStream();
			final DigestOutputStream dos = new DigestOutputStream(baos, digest);
			dos.write(data, off, len);
			dos.close();
			return digest.digest();
		} catch (IOException | NoSuchAlgorithmException e) {
			throw new IllegalStateException(e);
		}
	}

	public static byte[] hash(InputStream in, String algorithm) {
		try {
			final MessageDigest digest = MessageDigest.getInstance(algorithm);
			final ByteArrayOutputStream baos = new ByteArrayOutputStream();
			final DigestOutputStream dos = new DigestOutputStream(baos, digest);
			copy(in, dos);
			dos.close();
			return digest.digest();
		} catch (IOException | NoSuchAlgorithmException e) {
			throw new IllegalStateException(e);
		}
	}

	public static long copy(InputStream src, OutputStream dst) throws IOException {
		final byte[] buf = new byte[4 * 1024];
		long sum = 0;
		for (int got; (got = src.read(buf)) != -1; sum += got)
			dst.write(buf, 0, got);
		return sum;
	}
}
