package nl.west.rme.common.util;

import com.google.gson.GsonBuilder;
import com.google.gson.JsonElement;
import com.google.gson.JsonNull;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.google.gson.JsonPrimitive;
import java.util.Map;
import java.util.Map.Entry;


public class JsonUtils
{
  public static String toJson(JsonElement elem)
  {
    if (elem == null)
      return "null";
    return new GsonBuilder()
        .serializeNulls().setPrettyPrinting().disableHtmlEscaping().create()
        .toJson(elem);
  }

  public static JsonElement fromJson(String elem)
  {
    if (elem == null)
      return null; // this is debatable
    return new JsonParser().parse(elem);
  }

  public static JsonObject createJsonObjectFromMap(Map<String, String> map)
  {
    final JsonObject obj = new JsonObject();
    for (final Entry<String, String> entry : map.entrySet())
      if (entry.getValue() == null)
        obj.add(entry.getKey(), JsonNull.INSTANCE);
      else
        obj.addProperty(entry.getKey(), entry.getValue());
    return obj;
  }

  public static final String getAsString(JsonObject json, String prop)
  {
    final JsonElement elem = json.get(prop);
    if (elem == null)
      return null;
    if (elem == JsonNull.INSTANCE)
      return null;
    if (elem instanceof JsonPrimitive)
      return elem.getAsString();
    return null;
  }

  public static final JsonObject getAsJsonObject(JsonObject json, String prop)
  {
    final JsonElement elem = json.get(prop);
    if (elem == null)
      return null;
    if (elem == JsonNull.INSTANCE)
      return null;
    if (elem instanceof JsonObject)
      return elem.getAsJsonObject();
    return null;
  }
}
