package nl.west.rme.common.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;


public class MapList<K, V>
{
  private final Map<K, List<V>> data;

  public MapList()
  {
    this.data = this.createParent();
  }

  //

  public Set<K> keys()
  {
    return data.keySet();
  }

  public Collection<List<V>> values()
  {
    return data.values();
  }

  //

  public boolean add(K key, V value)
  {
    List<V> child = data.get(key);
    if (child == null)
      data.put(key, child = this.createChild());
    return child.add(value);
  }

  public boolean contains(K key, V value)
  {
    final List<V> child = data.get(key);
    return (child == null) ? false : child.contains(value);
  }

  public boolean remove(K key, V value)
  {
    final List<V> child = data.get(key);
    final boolean isRemoved = (child == null) ? false : child.remove(value);
    if (isRemoved && child.isEmpty())
      data.remove(key);
    return isRemoved;
  }

  //

  public List<V> currentChild(K key)
  {
    return data.get(key);
  }

  public List<V> ensureChild(K key)
  {
    List<V> child = data.get(key);
    if (child == null)
      data.put(key, child = this.createChild());
    return child;
  }

  //

  protected Map<K, List<V>> createParent()
  {
    return new HashMap<K, List<V>>();
  }

  protected List<V> createChild()
  {
    return new ArrayList<V>();
  }
}
