package nl.west.rme.common.util;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;


public class MapMap<G, K, V>
{
  public static void main(String[] args)
  {
    final MapMap<String, String, String> map = new MapMap<>();

    map.put("a", "0", "_");
    map.put("a", "1", "-");
    map.put("b", "0", "_");
    map.put("b", "1", "-");

    for (final MapMapEntry<String, String, String> entry : map.entries()) {
      System.out.println(entry);
    }
  }

  private final Map<G, Map<K, V>> data;

  public MapMap()
  {
    this.data = this.createParent();
  }

  public Map<G, Map<K, V>> backing()
  {
    return data;
  }

  //

  public Set<G> groups()
  {
    return data.keySet();
  }

  public Set<K> keys(G group)
  {
    final Map<K, V> child = data.get(group);
    if (child == null)
      return null;
    return child.keySet();
  }

  public void putAll(MapMap<G, K, V> that)
  {
    for (final MapMapEntry<G, K, V> entry : that.entries()) {
      this.put(entry.getGroup(), entry.getKey(), entry.getValue());
    }
  }

  public V put(G group, K key, V value)
  {
    Map<K, V> child = data.get(group);
    if (child == null)
      data.put(group, child = this.createChild());
    return child.put(key, value);
  }

  public V get(G group, K key)
  {
    final Map<K, V> child = data.get(group);
    return (child == null) ? null : child.get(key);
  }

  public V get(G group, K key, V fallback)
  {
    final Map<K, V> child = data.get(group);
    if (child == null || !child.containsKey(key))
      return fallback;
    return child.get(key);
  }

  public boolean contains(G group, K key)
  {
    final Map<K, V> child = data.get(group);
    return (child == null) ? false : child.containsKey(key);
  }

  public V remove(G group, K key)
  {
    final Map<K, V> child = data.get(group);
    final V removed = (child == null) ? null : child.remove(key);
    if (child != null && child.isEmpty())
      data.remove(group);
    return removed;
  }

  //

  public boolean containsGroup(G group)
  {
    return data.containsKey(group);
  }

  public Map<K, V> currentGroup(G group)
  {
    return data.get(group);
  }

  public Map<K, V> ensureGroup(G group)
  {
    Map<K, V> child = data.get(group);
    if (child == null)
      data.put(group, child = this.createChild());
    return child;
  }

  //

  protected Map<G, Map<K, V>> createParent()
  {
    return new HashMap<G, Map<K, V>>();
  }

  protected Map<K, V> createChild()
  {
    return new HashMap<K, V>();
  }

  public static class MapMapEntry<G, K, V>
  {
    private final G group;
    private final K key;
    private final V value;

    public MapMapEntry(G group, K key, V value)
    {
      this.group = group;
      this.key = key;
      this.value = value;
    }

    public G getGroup()
    {
      return group;
    }

    public K getKey()
    {
      return key;
    }

    public V getValue()
    {
      return value;
    }

    @Override
    public String toString()
    {
      return "[group=" + group + ", key=" + key + ", value=" + value + "]";
    }
  }

  public Iterable<MapMapEntry<G, K, V>> entries()
  {
    final IterUtils.IterFetchStream<MapMapEntry<G, K, V>> stream =
      new IterUtils.IterFetchStream<MapMapEntry<G, K, V>>()
      {
        private final Iterator<Map.Entry<G, Map<K, V>>> groupIterator = data
            .entrySet().iterator();
        private Map.Entry<G, Map<K, V>> groupEntry;
        private Iterator<Map.Entry<K, V>> childIterator;

        @Override
        public MapMapEntry<G, K, V> fetch()
        {
          for (;;) {
            if (childIterator == null) {
              if (!groupIterator.hasNext())
                return null;
              groupEntry = groupIterator.next();
              childIterator = groupEntry.getValue().entrySet().iterator();
            }

            if (childIterator.hasNext()) {
              final Map.Entry<K, V> childEntry = childIterator.next();
              return new MapMapEntry<G, K, V>(
                  groupEntry.getKey(), childEntry.getKey(),
                  childEntry.getValue());
            }
            else {
              childIterator = null;
            }
          }
        }
      };

    return IterUtils.iterable(stream);
  }

  @Override
  public String toString()
  {
    final StringBuilder sb = new StringBuilder();
    sb.append("[");
    int counter = 0;
    for (final MapMapEntry<G, K, V> entry : this.entries()) {
      sb.append("\n\t{ group=");
      sb.append(entry.getGroup()).append(", key=");
      sb.append(entry.getKey()).append(", value=");
      sb.append(entry.getValue());
      sb.append("},");
      counter++;
    }
    if (counter > 0)
      sb.append("\n");
    sb.append("]");
    return sb.toString();
  }
}
