package nl.west.rme.common.util;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;


public class MapMapSet<G, K, V>
{
  public static void main(String[] args)
  {
    final MapMapSet<String, String, String> map = new MapMapSet<>();

    map.add("a", "0", "_");
    map.add("a", "1", "-");
    map.add("a", "0", "_1");
    map.add("a", "1", "-2");
    map.add("b", "0", "_");
    map.add("b", "1", "-");
  }

  private final Map<G, Map<K, Set<V>>> data;

  public MapMapSet()
  {
    this.data = this.createGroupMap();
  }

  public Map<G, Map<K, Set<V>>> backing()
  {
    return data;
  }

  //

  public boolean add(G group, K key, V value)
  {
    Map<K, Set<V>> map = data.get(group);
    if (map == null)
      data.put(group, map = this.createKeyValueMap());

    Set<V> set = map.get(key);
    if (set == null)
      map.put(key, set = this.createValueSet());

    return set.add(value);
  }

  public Set<G> keys()
  {
    return data.keySet();
  }

  public Map<K, Set<V>> get(G group)
  {
    return data.get(group);
  }

  public Set<V> get(G group, K key)
  {
    final Map<K, Set<V>> map = data.get(group);
    return (map == null) ? null : map.get(key);
  }

  public Set<V> get(G group, K key, Set<V> fallback)
  {
    final Map<K, Set<V>> map = data.get(group);
    if (map == null || !map.containsKey(key))
      return fallback;
    return map.get(key);
  }

  public boolean contains(G group, K key, V value)
  {
    final Map<K, Set<V>> map = data.get(group);
    if (map == null)
      return false;

    final Set<V> set = map.get(key);
    if (set == null)
      return false;

    return set.contains(key);
  }

  public boolean remove(G group, K key, V value)
  {
    final Map<K, Set<V>> map = data.get(group);
    if (map == null)
      return false;

    final Set<V> set = map.get(key);
    if (set == null)
      return false;

    final boolean isRemoved = set.remove(value);
    if (!isRemoved)
      return false;

    if (set.isEmpty())
      map.remove(key);
    if (map.isEmpty())
      data.remove(group);

    return isRemoved;
  }

  //

  public boolean containsGroup(G group)
  {
    return data.containsKey(group);
  }

  public Map<K, Set<V>> currentGroup(G group)
  {
    return data.get(group);
  }

  public Map<K, Set<V>> ensureGroup(G group)
  {
    Map<K, Set<V>> child = data.get(group);
    if (child == null)
      data.put(group, child = this.createKeyValueMap());
    return child;
  }

  //

  protected Map<G, Map<K, Set<V>>> createGroupMap()
  {
    return new HashMap<G, Map<K, Set<V>>>();
  }

  protected Map<K, Set<V>> createKeyValueMap()
  {
    return new HashMap<K, Set<V>>();
  }

  protected Set<V> createValueSet()
  {
    return new HashSet<V>();
  }

  public static class MapMapSetEntry<G, K, V>
  {
    private final G group;
    private final K key;
    private final V value;

    public MapMapSetEntry(G group, K key, V value)
    {
      this.group = group;
      this.key = key;
      this.value = value;
    }

    public G getGroup()
    {
      return group;
    }

    public K getKey()
    {
      return key;
    }

    public V getValue()
    {
      return value;
    }

    @Override
    public String toString()
    {
      return "[group=" + group + ", key=" + key + ", value=" + value + "]";
    }
  }
}
