package nl.west.rme.common.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;


public class Measure implements AutoCloseable
{
  private static class Log
  {
    Map<String, List<Long>> name2durations = new HashMap<>();

    private void add(Measure m)
    {
      final long elapsed = System.nanoTime() - m.started;
      List<Long> durations = name2durations.get(m.name);
      if (durations == null)
        name2durations.put(m.name, durations = new ArrayList<>());
      durations.add(Long.valueOf(elapsed));
    }

    private void print()
    {
      int totalCount = 0;
      int totalSum = 0;
      for (final Entry<String, List<Long>> entry : name2durations.entrySet()) {
        final int count = entry.getValue().size();
        if (count == 0)
          continue;

        long sum = 0;
        long avg = 0;
        long min = Long.MAX_VALUE;
        long max = Long.MIN_VALUE;
        for (final Long v : entry.getValue()) {
          final long val = v.longValue();
          min = Math.min(min, val);
          max = Math.max(max, val);
          sum += val;
        }
        avg = sum / count;

        sum = (sum / 1_000_000L);
        avg = (avg / 1_000_000L);
        min = (min / 1_000_000L);
        max = (max / 1_000_000L);

        final String add =
          (count == 1) ? "" : ", min/avg/max: " + min + "/" + avg + "/" + max
              + "ms";
        log.Log.debug(entry.getKey() + ": " + count + "x sum=" + sum + "ms"
            + add);

        totalCount += count;
        totalSum += sum;
      }
      if (totalCount > 0) {
        log.Log.debug("totals: " + totalCount + "x sum=" + totalSum
            + "ms, avg=" + (totalSum / totalCount) + "ms");
      }
      name2durations.clear();
    }
  }

  private static ThreadLocal<Log> local = new ThreadLocal<Log>()
  {
    @Override
    protected Log initialValue()
    {
      return new Log();
    }
  };

  public static void print()
  {
    local.get().print();
  }

  public static Measure tryWith(String name)
  {
    return new Measure(name);
  }

  private final String name;
  private final long started;

  private Measure(String name)
  {
    this.name = name;
    this.started = System.nanoTime();
  }

  @Override
  public void close()
  {
    local.get().add(this);
  }
}
