package nl.west.rme.common.util;

import log.Log;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;
import javax.servlet.http.HttpServletRequest;

public class RegexRules {
	private final String debugName;
	private final List<Pattern> includePatterns;
	private final List<Pattern> excludePatterns;

	public RegexRules() {
		this(null);
	}

	public RegexRules(String debugName) {
		this.debugName = debugName;
		includePatterns = new ArrayList<>();
		excludePatterns = new ArrayList<>();
	}

	public void parseRules(String rules) {
		for (String rule : Text.splitOnLines(rules)) {
			rule = Text.trimToNull(rule);
			if (rule == null)
				continue;

			if (rule.startsWith("include:"))
				this.addIncludePattern(rule.substring("include:".length()).trim());
			else if (rule.startsWith("exclude:"))
				this.addExcludePattern(rule.substring("exclude:".length()).trim());
			else
				throw new IllegalArgumentException("prefix ('include:' / 'exclude:') required for rule: " + rule);
		}
	}

	public void addIncludePattern(String pattern) {
		includePatterns.add(Pattern.compile(pattern));
	}

	public void addExcludePattern(String pattern) {
		excludePatterns.add(Pattern.compile(pattern));
	}

	public boolean matchesURI(HttpServletRequest request) {
		// strip context-path from URI
		String uri = request.getRequestURI();
		uri = uri.substring(request.getContextPath().length());
		return this.matches(uri);
	}

	private boolean matches(String s) {
		final Map<String, Boolean> uri2match = uri2match_localCache.get();
		Boolean isMatch = uri2match.get(s);
		if (isMatch == null) {
			isMatch = Boolean.valueOf(this.matchesImpl(s));

			uri2match.put(s, isMatch);
			if (uri2match.size() > 1024)
				uri2match.clear();
		}
		if (debugName != null)
			Log.info(this.getClass().getSimpleName() + "[" + debugName + "] '" + s + "' --> " + isMatch);
		return isMatch.booleanValue();
	}

	public boolean matchesImpl(String s) {
		return (includePatterns.isEmpty() || this.matchesRegex(s, includePatterns)) && //
				!this.matchesRegex(s, excludePatterns);
	}

	private boolean matchesRegex(String s, List<Pattern> patterns) {
		for (final Pattern pattern : patterns) {
			if (pattern.matcher(s).matches()) {
				return true;
			}
		}
		return false;
	}

	private final WebContextThreadLocal<Map<String, Boolean>> uri2match_localCache = //
	new WebContextThreadLocal<Map<String, Boolean>>() {
		@Override
		protected Map<String, Boolean> initialValue() {
			return new HashMap<>();
		}
	};
}
