package nl.west.rme.common.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;


public class SetOperations
{
  public static <T> Collection<T> orderBy(
      Collection<T> values, Comparator<T> orderBy)
  {
    final List<T> list = new ArrayList<T>(values);
    Collections.sort(list, orderBy);
    return list;
  }

  //

  public static <T> Set<T> and(Set<T> a, Set<T> b)
  {
    final Set<T> c = new HashSet<>();
    c.addAll(a);
    c.retainAll(b);
    return c;
  }

  //

  public static <T> Set<T> findAdded(Set<T> older, Set<T> newer)
  {
    final Set<T> removed = new HashSet<>();
    removed.addAll(newer);
    removed.removeAll(older);
    return removed;
  }

  public static <T> Set<T> findRetained(Set<T> a, Set<T> b)
  {
    return and(a, b);
  }

  public static <T> Set<T> findRemoved(Set<T> older, Set<T> newer)
  {
    final Set<T> removed = new HashSet<>();
    removed.addAll(older);
    removed.removeAll(newer);
    return removed;
  }

  //

  public static <T> ChangeSet<T> createChangeSet(Set<T> older, Set<T> newer)
  {
    final ChangeSet<T> changeSet = new ChangeSet<>();
    changeSet.older = older;
    changeSet.newer = newer;
    changeSet.added = findAdded(older, newer);
    changeSet.retained = findRetained(older, newer);
    changeSet.removed = findRemoved(older, newer);
    return changeSet;
  }

  public static class ChangeSet<T>
  {
    public Set<T> older;
    public Set<T> newer;

    public Set<T> added;
    public Set<T> retained;
    public Set<T> removed;
  }
}
