package nl.west.rme.common.util;

import log.Log;
import java.util.Calendar;


public class TextTime // "HH:MM:SS"
{
  private final Calendar calendar;

  public TextTime(Calendar calendar)
  {
    this.calendar = calendar;
  }

  public String now()
  {
    return format(System.currentTimeMillis());
  }

  public String format(long timestamp)
  {
    calendar.setTimeInMillis(timestamp);
    return format(//
        calendar.get(Calendar.HOUR_OF_DAY), //
        calendar.get(Calendar.MINUTE),//
        calendar.get(Calendar.SECOND));
  }

  public static int toSeconds(String time)
  {
    verify(time);

    final String[] parts = time.split(":");
    final int hh = Integer.parseInt(parts[0]);
    final int mm = Integer.parseInt(parts[1]);
    final int ss = Integer.parseInt(parts[2]);

    final int seconds = (hh * 3600 + mm * 60 + ss);
    if (seconds < 0 || seconds >= 86400)
      throw new IllegalStateException("day overflow: resulting seconds="
          + seconds);
    return seconds;
  }

  public static String fromSeconds(int seconds)
  {
    if (seconds < 0 || seconds >= 86400)
      throw new IllegalStateException("day overflow: resulting seconds="
          + seconds);

    final int hh = seconds / 3600;
    final int mm = seconds / 60 % 60;
    final int ss = seconds % 60;

    return format(hh, mm, ss);
  }

  public static String addSeconds(String time, int seconds)
  {
    return fromSeconds(toSeconds(time) + seconds);
  }

  //

  public static boolean isValid(String time)
  {
    if (time == null)
      return false;
    final String[] parts = time.split(":");
    if (parts.length != 3)
      return false;

    int hh, mm, ss;
    try {
      hh = Integer.parseInt(parts[0]);
      mm = Integer.parseInt(parts[1]);
      ss = Integer.parseInt(parts[2]);
    }
    catch (final NumberFormatException exc) {
      return false;
    }

    if (hh < 0 || hh >= 24)
      return false;
    if (mm < 0 || mm >= 60)
      return false;
    if (ss < 0 || ss >= 60)
      return false;
    return true;
  }

  public static String verify(String time)
  {
    if (!isValid(time))
      throw new IllegalArgumentException("not a valid time: " + time);
    return time;
  }

  public static String parseTime(String time, String orElse)
  {
    if (isValid(time))
      return time;
    Log.warn("failed to parse time: [" + time + "]");
    return orElse;
  }

  public static String format(int hh, int mm, int ss)
  {
    final StringBuilder sb = new StringBuilder();
    sb.append(format(hh, 2));
    sb.append(':');
    sb.append(format(mm, 2));
    sb.append(':');
    sb.append(format(ss, 2));
    return sb.toString();
  }

  public static String format(Calendar calendar)
  {
    return format(//
        calendar.get(Calendar.HOUR_OF_DAY), //
        calendar.get(Calendar.MINUTE),//
        calendar.get(Calendar.SECOND));
  }

  public static void apply(String time, Calendar calendar)
  {
    verify(time);

    final String[] parts = time.split(":");
    final int hh = Integer.parseInt(parts[0]);
    final int mm = Integer.parseInt(parts[1]);
    final int ss = Integer.parseInt(parts[2]);

    calendar.set(Calendar.HOUR_OF_DAY, hh);
    calendar.set(Calendar.MINUTE, mm);
    calendar.set(Calendar.SECOND, ss);
    calendar.set(Calendar.MILLISECOND, 0);
  }

  private static String format(int value, int len)
  {
    final StringBuilder sb = new StringBuilder();
    sb.append(value);
    while (sb.length() < len)
      sb.insert(0, '0');
    return sb.toString();
  }
}
